package workers

import (
	"fmt"
	"time"
	"ORsearch/consts"
	"ORsearch/structures"
)



// A worker receives a branch with a matrix of size consts.DISTRIBUTIONROWS and launches the branch search from there
// It records its best result
func worker(startinglevels []uint16, freethread chan structures.Empty, res chan structures.SearchResult) {

	if consts.DISPLAY >= 2 { fmt.Println(startinglevels, "started at", time.Now()) }

	// *** INITIALIZATION ***

	// The result variables
	var bestlength = consts.DISTRIBUTIONROWS-1
	var bestmatrix = structures.NewMatrix(structures.ConfigureM(consts.N), consts.N)

	// The matrix, its compatible pairs and the symmetry
	var m = structures.InitializeMatrix(startinglevels)
	var compatiblepairs, cardinalityQr, goodbranch = structures.InitCompatiblePairs(m, consts.DISTRIBUTIONROWS-1)
	// We only initialize for the consts.DISTRIBUTIONROWS-1 first rows, the last one being settled by branchsearch()
	var sym = structures.InitSymmetry(m, consts.DISTRIBUTIONROWS)
	// Note that we could have gathered the compatible pairs and symmetry information from StartWorkers()
	// but we decided that the time improvement was not worth the additional complexity in the code

	// We have to initialize brancheslist even though it is useless here
	var brancheslist = make([][]uint16, 0)

	// *** LAUNCH THE SEARCH ***

	if goodbranch {
		branchSearch(consts.DISTRIBUTIONROWS, m, compatiblepairs, cardinalityQr, &bestlength, bestmatrix, 0, &brancheslist, sym)
		if consts.DISPLAY >= 2 { fmt.Println(startinglevels, "found", bestlength, "finished at", time.Now()) }
	} else {
		if consts.DISPLAY >= 2 { fmt.Println(startinglevels, "skipped at", time.Now()) }
	}

	// *** SEND THE RESULTS ***
	
	res <- structures.SearchResult{startinglevels, bestlength, bestmatrix} 

	// Make sure it is happening at the end... where it is actually, but who knows
	freethread <- structures.Empty{} // Signal to show worker has finished

}


